<?php
/**
 * The Horde_Cache_xcache:: class provides an XCache implementation of
 * the Horde caching system.
 *
 * $Horde: framework/Cache/Cache/xcache.php,v 1.2 2007/03/08 23:17:13 slusarz Exp $
 *
 * Copyright 2006 Duck <duck@obala.net>
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @since   Horde 4.0
 * @package Horde_Cache
 */
class Horde_Cache_xcache extends Horde_Cache {

    /**
     * Attempts to retrieve a piece of cached data and return it to the caller.
     *
     * @param string  $key       Cache key to fetch.
     * @param integer $lifetime  Lifetime of the key in seconds.
     *
     * @return mixed  Cached data, or false if none was found.
     */
    function get($key, $lifetime = 1)
    {
        $this->_setExpire($key, $lifetime);
        return xcache_get($key);
    }

    /**
     * Attempts to store an object to the cache.
     *
     * @param string $key   Cache key (identifier).
     * @param mixed  $data  Data to store in the cache.
     *
     * @return boolean  True on success, false on failure.
     */
    function set($key, $data)
    {
        xcache_set($key . '_expire', time(), 0);
        return xcache_set($key, $data, 0);
    }

    /**
     * Attempts to directly output cached data.
     *
     * @param string  $key       Cache key to output.
     * @param integer $lifetime  Lifetime of the key in seconds.
     *
     * @return boolean  True if output or false if no object was found.
     */
    function output($key, $lifetime = 1)
    {
        $data = $this->get($key, $lifetime);
        if ($data !== false) {
            echo $data;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Checks if a given key exists in the cache, valid for the given lifetime.
     *
     * @param string  $key       Cache key to check.
     * @param integer $lifetime  Lifetime of the key in seconds.
     *
     * @return boolean  Existance.
     */
    function exists($key, $lifetime = 1)
    {
        return xcache_isset($key);
    }

    /**
     * Expire any existing data for the given key.
     *
     * @param string $key  Cache key to expire.
     *
     * @return boolean  Success or failure.
     */
    function expire($key)
    {
        xcache_unset($key . '_expire');
        return xcache_unset($key);
    }

    /**
     * Set expire time on each call since memcache sets it on cache creation.
     *
     * @access private
     *
     * @param string  $key   Cache key to expire.
     * @param integer $lifetime  Lifetime of the data in seconds.
     */
    function _setExpire($key, $lifetime)
    {
        if ($lifetime == 0) {
            // don't expire
            return true;
        }

        $expire = xcache_get($key . '_expire');
        if ($expire == 0) {
            // alreay checked
            return true;
        }

        // set prune period
        if ($expire+$lifetime<time()) {
            $lifetime = 0;
        } else {
            xcache_set($key . '_expire', 0);
            $lifetime = ($expire+$lifetime) - time();
        }

        xcache_unset($key);
        xcache_unset($key . '_expire');

        return $lifetime;
    }

}
